(function () {
'use strict';

function boundingBox(imageData){
    var p = imageData.data;
    var w = imageData.width;
    var h = imageData.height;

    var top = 0;
top:
    for(var y = 0; y < h; ++y){
        for(var x = 0; x < w; ++x){
            var c = (y * w + x) * 4;
            if(p[c+3] !== 0)break top;
        }
        top = y;
    }

    var bottom = h;
bottom:
    for(var y$1 = h-1; y$1 >= 0; --y$1){
        for(var x$1 = 0; x$1 < w; ++x$1){
            var c$1 = (y$1 * w + x$1) * 4;
            if(p[c$1+3] !== 0)break bottom;
        }
        bottom = y$1;
    }

    var left = 0;
left:
    for(var x$2 = 0; x$2 < w; ++x$2){
        for(var y$2 = 0; y$2 < h; ++y$2){
            var c$2 = (y$2 * w + x$2) * 4;
            if(p[c$2+3] !== 0)break left;
        }
        left = x$2;
    }

    var right = w;
right:
    for(var x$3 = w-1; x$3 >= 0; --x$3){
        for(var y$3 = 0; y$3 < h; ++y$3){
            var c$3 = (y$3 * w + x$3) * 4;
            if(p[c$3+3] !== 0)break right;
        }
        right = x$3;
    }

    return {
        x: left,
        y: top,
        width: right - left,
        height: bottom - top,
    };
}

function load(imagePath){
    var image = new Image();
    image.src = imagePath;

    return new Promise(function (resolve, reject){
        image.onload = function (){ return resolve(image); };
        image.onerror = function (e){ return reject(e); };
    });
}

function toArrayBuffer(canvas){
    return new Promise(function (resolve, reject){
        var binStr = atob( canvas.toDataURL().split(',')[1] ),
            len = binStr.length,
            arr = new Uint8Array(len);

        for (var i=0; i<len; i++ ) {
            arr[i] = binStr.charCodeAt(i);
        }

        resolve(arr);
    });
}


function currentDir(){
    var fileName = global.require.main.filename;
    return fileName.split('/').slice(0, -1).join('/');
}

function trim(imagePath){
    var bounds = null;
    var originalSize = {};

    return load(imagePath).then(function (img){
        var src = document.createElement('canvas');
        src.width = img.width;
        src.height = img.height;
        var sc = src.getContext('2d');
        sc.drawImage(img, 0, 0);
        var imageData = sc.getImageData(0, 0, src.width, src.height);
        bounds = boundingBox(imageData);

        originalSize.width = img.width;
        originalSize.height = img.height;

        var out = document.createElement('canvas');
        out.width = bounds.width;
        out.height = bounds.height;
        var oc = out.getContext('2d');
        oc.drawImage(img, bounds.x, bounds.y, bounds.width, bounds.height, 0, 0, bounds.width, bounds.height);
        return toArrayBuffer(out);

    }).then(function (buffer){
        var fs = require('fs');
        var path = require('path');
        var fileName = imagePath.split('/').pop();
        var writePath = path.join(currentDir(), 'img', 'trim', fileName);
        return new Promise(function (resolve, reject){
            fs.writeFile(writePath, new Buffer(buffer), null, function (err){
                if(err) reject(err);
                resolve({fileName: fileName, bounds: bounds, originalSize: originalSize});
            });
        });
    });
}

function installHooks(){
    DataManager._databaseFiles.push({
        name: '$dataTrim',
        src: 'Trim.json'
    });

    var Sprite_Picture_initialize = Sprite_Picture.prototype.initialize;
    Sprite_Picture.prototype.initialize = function(pictureId){
        this._spritePicture = new Sprite();
        Sprite_Picture_initialize.call(this, pictureId);
        this.addChild(this._spritePicture);
    };

    var Sprite_Picture_loadBitmap = Sprite_Picture.prototype.loadBitmap;
    Sprite_Picture.prototype.loadBitmap = function() {
        var sprite = this._spritePicture;

        if($dataTrim[this._pictureName]){
            this.bitmap = null;

            var trim = $dataTrim[this._pictureName];
            var bitmap = ImageManager.loadBitmap('img/trim/', this._pictureName, null, true);
            sprite.x = trim.bounds.x;
            sprite.y = trim.bounds.y;
            sprite.bitmap = bitmap;
            // sprite.bitmap.addLoadListener(()=>{
            //     if(bitmap === sprite.bitmap){
            //         //it does not need 1.5~
            //         sprite.setFrame(0, 0, 0, 0);
            //         sprite._refresh();
            //     }
            // });
            sprite.visible = true;
            this.visible = true;
            this._trim = trim;
        }else{
            this._trim = null;
            sprite.visible = false;
            Sprite_Picture_loadBitmap.call(this);
        }
    };

    var Sprite_Picture_updateOrigin = Sprite_Picture.prototype.updatePosition;
    Sprite_Picture.prototype.updateOrigin = function() {
        var picture = this.picture();
        var trim = this._trim;
        if(trim){
            if (picture.origin() === 0) {
                this.pivot.set(0,0);
            } else {
                var vw = trim.originalSize.width;
                var vh  =trim.originalSize.height;
                this.pivot.set(
                    vw * 0.5,
                    vh * 0.5
                    );
            }
        }else{
            Sprite_Picture_updateOrigin.call(this);
        }
    };


    //
    // const Sprite_Picture_updatePosition = Sprite_Picture.prototype.updatePosition;
    // Sprite_Picture.prototype.updatePosition = function(){
    //     Sprite_Picture_updatePosition.call(this);
    //
    //     const geom = $dataTrim[this.picture().name()];
    //     this.x += geom.bounds.x;
    //     this.y += geom.bounds.y;
    // };
}

function enumPictures(){
    var fs = require('fs');
    var path = require('path');

    return new Promise(function (resolve, reject){
        fs.readdir(path.join(currentDir(), 'img', 'pictures'), function (err, files){
            if(err)reject(err);

            var fileNames = files.map(function (file){ return path.basename(file); });
            resolve(fileNames);
        });
    });
}

function trimImages(){
    var fs = require('fs');
    var path = require('path');

    var trimDir = path.join(currentDir(), 'img', 'trim');
    if(!fs.existsSync(trimDir)){
        fs.mkdirSync(trimDir);
    }

    return enumPictures().then(function (images){
        return Promise.all(
            images.map(function (image){ return trim(("./img/pictures/" + image)); })
        );
    }).then(function (trim){
        var json = {};
        trim.map(function (geom){
            json[path.basename(geom.fileName, '.png')] = {
                bounds: geom.bounds,
                originalSize: geom.originalSize
            };
        });

        return new Promise(function (resolve, reject){
            fs.writeFile(path.join(currentDir(), 'data', 'Trim.json'), JSON.stringify(json), function (err){
                if(err)reject(err);
                resolve();
            });
        });
    });
}

if(Utils.isOptionValid('test')){
    var fs = require('fs');
    var path = require('path');
    if(!fs.existsSync(path.join(currentDir(), 'data', 'Trim.json'))){
        alert('切り取り処理を開始します');
        trimImages().then(function (){ return alert('切り取り処理を終了しました。再度実行するにはdata/Trim.jsonを削除してください。'); });
    }else{
        installHooks();
    }
}else{
    installHooks();
}

}());